;!function(){try{var k="_btDebugIds",u="undefined",v="ee0c7e7b-2ad7-692f-8baf-c6f1243884ac",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
/* eslint-disable no-fallthrough */
import * as fs from 'fs';
import { iCPSEventLog, iCPSEventRuntimeError, iCPSEventRuntimeWarning, iCPSEventSyncEngine } from "../../lib/resources/events-types.js";
import { Resources } from "../../lib/resources/main.js";
import { FILE_ENCODING } from '../../lib/resources/resource-types.js';
import { iCPSError } from "../error/error.js";
export var LogLevel;
(function (LogLevel) {
    LogLevel["DEBUG"] = "debug";
    LogLevel["INFO"] = "info";
    LogLevel["WARN"] = "warn";
    LogLevel["ERROR"] = "error";
})(LogLevel || (LogLevel = {}));
/**
 * This class handles the input/output to a log file
 */
export class LogInterface {
    /**
     * The opened file descriptor of the log file
     */
    logFileDescriptor;
    constructor() {
        // Try opening the file - truncate if exists
        this.logFileDescriptor = fs.openSync(Resources.manager().logFilePath, `w`);
        switch (Resources.manager().logLevel) {
            case `debug`:
                Resources.events(this).on(iCPSEventLog.DEBUG, (source, msg) => this.logMessage(LogLevel.DEBUG, source, msg));
            default:
            case `info`:
                Resources.events(this).on(iCPSEventLog.INFO, (source, msg) => this.logMessage(LogLevel.INFO, source, msg));
            case `warn`:
                Resources.events(this)
                    .on(iCPSEventLog.WARN, (source, msg) => this.logMessage(LogLevel.WARN, source, msg))
                    .on(iCPSEventRuntimeWarning.COUNT_MISMATCH, (album, expectedCount, actualCPLAssets, actualCPLMasters) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Expected ${expectedCount} CPLAssets & CPLMasters, but got ${actualCPLAssets} CPLAssets and ${actualCPLMasters} CPLMasters for album ${album}`);
                })
                    .on(iCPSEventRuntimeWarning.FILETYPE_ERROR, (ext, descriptor) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Unknown file extension ${ext} for descriptor ${descriptor}`);
                })
                    .on(iCPSEventRuntimeWarning.LIBRARY_LOAD_ERROR, (err, filePath) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error while loading file ${filePath}: ${iCPSError.toiCPSError(err).getDescription()}`);
                })
                    .on(iCPSEventRuntimeWarning.EXTRANEOUS_FILE, (filePath) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Extraneous file found in directory ${filePath}`);
                })
                    .on(iCPSEventRuntimeWarning.ICLOUD_LOAD_ERROR, (err, asset) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error while loading iCloud asset ${asset.recordName}: ${iCPSError.toiCPSError(err).getDescription()}`);
                })
                    .on(iCPSEventRuntimeWarning.WRITE_ASSET_ERROR, (err, asset) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error while verifying asset ${asset.getDisplayName()}: ${iCPSError.toiCPSError(err).getDescription()}`);
                })
                    .on(iCPSEventRuntimeWarning.WRITE_ALBUM_ERROR, (err, album) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error while writing album ${album.getDisplayName()}: ${iCPSError.toiCPSError(err).getDescription()}`);
                })
                    .on(iCPSEventRuntimeWarning.LINK_ERROR, (err, srcPath, dstPath) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error while linking ${srcPath} to ${dstPath}: ${iCPSError.toiCPSError(err).getDescription()}`);
                })
                    .on(iCPSEventRuntimeWarning.MFA_ERROR, (err) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error within MFA flow: ${err.getDescription()}`);
                })
                    .on(iCPSEventRuntimeWarning.RESOURCE_FILE_ERROR, (err) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error while accessing resource file: ${iCPSError.toiCPSError(err).getDescription()}`);
                })
                    .on(iCPSEventRuntimeWarning.ARCHIVE_ASSET_ERROR, (err, assetPath) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Error while archiving asset ${assetPath}: ${iCPSError.toiCPSError(err).getDescription()}`);
                });
                Resources.events(this)
                    .on(iCPSEventSyncEngine.RETRY, (_retryCount, err) => {
                    this.logMessage(LogLevel.WARN, `RuntimeWarning`, `Detected error during sync: ${iCPSError.toiCPSError(err).getDescription()}`);
                });
            case `error`:
                Resources.events(this)
                    .on(iCPSEventLog.ERROR, (source, msg) => this.logMessage(LogLevel.ERROR, source, msg))
                    .on(iCPSEventRuntimeError.HANDLED_ERROR, (err) => this.logMessage(LogLevel.ERROR, `RuntimeError`, err.getDescription()));
        }
    }
    /**
     * Logs a message to the log file
     * @param level - The log level
     * @param source - The source of the message, either a string of an object instance
     * @param msg - The message to log
     */
    logMessage(level, source, msg) {
        const _source = typeof source === `string` ? source : String(source.constructor.name);
        const prefixedMessage = `[${new Date().toISOString()}] ${level.toUpperCase()} ${_source}: ${msg}\n`;
        if (Resources.manager().logToCli) {
            console.log(prefixedMessage);
            return;
        }
        fs.appendFileSync(this.logFileDescriptor, prefixedMessage, { encoding: FILE_ENCODING });
    }
}
//# sourceMappingURL=log.js.map
//# debugId=ee0c7e7b-2ad7-692f-8baf-c6f1243884ac