;!function(){try{var k="_btDebugIds",u="undefined",v="84b95565-712f-bfa1-7c23-59698fb456eb",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
import { ENDPOINTS } from '../../resources/network-types.js';
/**
 * Indicating, which MFA method should be used
 */
var MFAMethodType;
(function (MFAMethodType) {
    MFAMethodType[MFAMethodType["DEVICE"] = 1] = "DEVICE";
    MFAMethodType[MFAMethodType["SMS"] = 2] = "SMS";
    MFAMethodType[MFAMethodType["VOICE"] = 3] = "VOICE";
})(MFAMethodType || (MFAMethodType = {}));
/**
 * A class to hold the status of the currently executed MFA method
 */
export class MFAMethod {
    /**
     * The type of MFA method used
     */
    type;
    /**
     * The id of the device used
     */
    numberId;
    /**
     * Creates a new MFAMethod object to hold status information
     * @param mfaMethod - The method to be used. Defaults to `device`
     * @param numberId - The number id used for sending sms or voice codes. Defaults to 1
     */
    constructor(mfaMethod = `device`, numberId = 1) {
        this.update(mfaMethod, numberId);
    }
    /**
     * Updates this object to the given method and number id
     * @param mfaMethod - The method to be used. Defaults to `device`
     * @param numberId - The number id used for sending sms or voice codes. Defaults to 1
     */
    update(mfaMethod = `device`, numberId = 1) {
        switch (mfaMethod) {
            case `sms`:
                this.type = MFAMethodType.SMS;
                this.numberId = numberId;
                break;
            case `voice`:
                this.type = MFAMethodType.VOICE;
                this.numberId = numberId;
                break;
            default:
            case `device`:
                this.type = MFAMethodType.DEVICE;
                this.numberId = undefined;
                break;
        }
    }
    /**
     *
     * @returns True, if the 'device' method is active
     */
    get isDevice() {
        return this.type === MFAMethodType.DEVICE;
    }
    /**
     *
     * @returns True, if the 'sms' method is active
     */
    get isSMS() {
        return this.type === MFAMethodType.SMS;
    }
    /**
     *
     * @returns True, if the 'voice' method is active
     */
    get isVoice() {
        return this.type === MFAMethodType.VOICE;
    }
    /**
     *
     * @returns A string representation of this object
     */
    toString() {
        switch (this.type) {
            case MFAMethodType.SMS:
                return `'SMS' (Number ID: ${this.numberId})`;
            case MFAMethodType.VOICE:
                return `'Voice' (Number ID: ${this.numberId})`;
            default:
            case MFAMethodType.DEVICE:
                return `'Device'`;
        }
    }
    /**
     *
     * @returns The appropriate URL endpoint for resending the code, given the currently selected MFA Method
     */
    getResendURL() {
        switch (this.type) {
            case MFAMethodType.VOICE:
            case MFAMethodType.SMS:
                return ENDPOINTS.AUTH.BASE + ENDPOINTS.AUTH.PATH.MFA.PHONE_RESEND;
            default:
            case MFAMethodType.DEVICE:
                return ENDPOINTS.AUTH.BASE + ENDPOINTS.AUTH.PATH.MFA.DEVICE_RESEND;
        }
    }
    /**
     *
     * @returns The appropriate payload for resending the code, given the currently selected MFA Method
     */
    getResendPayload() {
        switch (this.type) {
            case MFAMethodType.VOICE:
                return {
                    phoneNumber: {
                        id: this.numberId,
                    },
                    mode: `voice`,
                };
            case MFAMethodType.SMS:
                return {
                    phoneNumber: {
                        id: this.numberId,
                    },
                    mode: `sms`,
                };
            default:
            case MFAMethodType.DEVICE:
                return undefined;
        }
    }
    /**
     * Checks if the status code matches our expectation for a successful resend
     * @param status - The status code for the response received from the backend
     * @returns True, if the response was successful, based on the currently selected MFA Method
     */
    resendSuccessful(status) {
        switch (this.type) {
            case MFAMethodType.VOICE:
            case MFAMethodType.SMS:
                return status === 200;
            default:
            case MFAMethodType.DEVICE:
                return status === 202;
        }
    }
    /**
     * @param mfa - The MFA code, that should be send for validation
     * @returns The appropriate payload for entering the code, given the currently selected MFA Method
     */
    getEnterPayload(mfa) {
        switch (this.type) {
            case MFAMethodType.VOICE:
                return {
                    securityCode: {
                        code: `${mfa}`,
                    },
                    phoneNumber: {
                        id: this.numberId,
                    },
                    mode: `voice`,
                };
            case MFAMethodType.SMS:
                return {
                    securityCode: {
                        code: `${mfa}`,
                    },
                    phoneNumber: {
                        id: this.numberId,
                    },
                    mode: `sms`,
                };
            default:
            case MFAMethodType.DEVICE:
                return {
                    securityCode: {
                        code: `${mfa}`,
                    },
                };
        }
    }
    /**
     *
     * @returns The appropriate URL endpoint for entering the code, given the currently selected MFA Method
     */
    getEnterURL() {
        switch (this.type) {
            case MFAMethodType.VOICE:
            case MFAMethodType.SMS:
                return ENDPOINTS.AUTH.BASE + ENDPOINTS.AUTH.PATH.MFA.PHONE_ENTER;
            default:
            case MFAMethodType.DEVICE:
                return ENDPOINTS.AUTH.BASE + ENDPOINTS.AUTH.PATH.MFA.DEVICE_ENTER;
        }
    }
    /**
     * Checks if the status code matches our expectation for a successful enter
     * @param status - The status code for the response received from the backend
     * @returns True, if the response was successful, based on the currently selected MFA Method
     */
    enterSuccessful(status) {
        switch (this.type) {
            case MFAMethodType.VOICE:
            case MFAMethodType.SMS:
                return status === 200;
            default:
            case MFAMethodType.DEVICE:
                return status === 204;
        }
    }
}
//# sourceMappingURL=mfa-method.js.map
//# debugId=84b95565-712f-bfa1-7c23-59698fb456eb