;!function(){try{var k="_btDebugIds",u="undefined",v="3c67d18c-90e9-fda4-fb0f-b90fa94f8068",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
import path from 'path';
import { FileType } from './file-type.js';
import fs from 'fs/promises';
import { iCPSError } from '../../../app/error/error.js';
import { LIBRARY_ERR } from '../../../app/error/error-codes.js';
import { Zones } from '../../icloud/icloud-photos/query-builder.js';
import { PRIMARY_ASSET_DIR, SHARED_ASSET_DIR } from '../constants.js';
import { Resources } from '../../resources/main.js';
/**
 * Representing the possible asset types
 */
export var AssetType;
(function (AssetType) {
    /**
     * Shows that this is the original file
     */
    AssetType[AssetType["ORIG"] = 0] = "ORIG";
    /**
     * Shows that this is the latest edit
     */
    AssetType[AssetType["EDIT"] = 1] = "EDIT";
    /**
     * Shows that this is a live photo
     */
    AssetType[AssetType["LIVE"] = 2] = "LIVE";
})(AssetType || (AssetType = {}));
/**
 * This class represents an Asset in the Photo Library
 */
export class Asset {
    /**
     * Checksum of the asset
     */
    fileChecksum;
    /**
     * File size in bytes of this asset
     */
    size;
    /**
     * Modified timestamp as epoch timestamp (ms since epoch)
     */
    modified;
    /**
     * The file type of this asset
     */
    fileType;
    /**
     * Shows which version of the asset this is
     */
    assetType;
    /**
     * The original filename of this asset
     */
    origFilename;
    /**
     * The zone this file is belonging to
     */
    zone;
    /**
     * The wrapping key of this asset (unknown usage, taken from backend, only present if fetched from CPL)
     */
    wrappingKey;
    /**
     * The reference checksum of this asset (unknown usage, taken from backend, only present if fetched from CPL)
     */
    referenceChecksum;
    /**
     * The download URL of this asset (only present if fetched from CPL)
     */
    downloadURL;
    /**
     * Record name of the associated CPL Asset
     */
    recordName;
    /**
     * Flag, if this asset is favorite
     */
    isFavorite;
    /**
     * Creates a new Asset object
     * @param fileChecksum -
     * @param size -
     * @param fileType -
     * @param modified -
     * @param zone - Which zone is this asset belonging to
     * @param assetType - If this asset is the original or an edit
     * @param origFilename - The original filename, extracted from the parent object
     * @param wrappingKey -
     * @param referenceChecksum -
     * @param downloadURL -
     */
    constructor(fileChecksum, size, fileType, modified, zone, assetType, origFilename, wrappingKey, referenceChecksum, downloadURL, recordName, isFavorite) {
        this.fileChecksum = fileChecksum;
        this.size = size;
        this.fileType = fileType;
        this.modified = modified;
        this.zone = zone;
        this.assetType = assetType;
        this.origFilename = origFilename;
        this.wrappingKey = wrappingKey;
        this.referenceChecksum = referenceChecksum;
        this.downloadURL = downloadURL;
        this.recordName = recordName;
        this.isFavorite = isFavorite;
    }
    /**
     * Creates an Asset from the information provided by the backend
     * @param asset - The AssetID object returned from the backend
     * @param fileTypeDescriptor - The assetType string, describing the filetype
     * @param fileTypeExt - The assetTypes's extension as derived from the encoded filename
     * @param modified - The modified date as returned from the backend (in ms since epoch)
     * @param origFilename - The original filename, extracted from the parent object
     * @param assetType - If this asset is the original or an edit
     * @param zone - Specifies the zone this asset is belonging to
     * @returns An Asset based on the backend objects
     * @throws An iCPSError, if the asset file descriptor is not supported
     */
    static fromCPL(asset, fileTypeDescriptor, fileTypeExt, modified, origFilename, assetType, recordName, isFavorite, zone) {
        return new Asset(asset.fileChecksum, asset.size, FileType.fromAssetType(fileTypeDescriptor, fileTypeExt), modified, zone === `PrimarySync` ? Zones.Primary : Zones.Shared, assetType, origFilename, asset.wrappingKey, asset.referenceChecksum, asset.downloadURL, recordName, isFavorite === 1);
    }
    /**
     * Creates an Asset from a given file
     * @param fileName - The file name of the file
     * @param stats - The metadata associated with the file
     * @param zone - Specifies the zone this asset is belonging to
     * @returns An Asset based on the file information
     * @throws An iCPSError, if the asset file extension is not supported
     */
    static fromFile(fileName, stats, zone) {
        return new Asset(Buffer.from(path.basename(fileName, path.extname(fileName)), `base64url`).toString(`base64`), stats.size, FileType.fromExtension(path.extname(fileName)), stats.mtimeMs, zone);
    }
    /**
     * Compares the provided asset to this asset instance
     * @param asset - The asset to compare to
     * @returns True if provided asset matches this instance (based on fileChecksum, fileType, size and modified timestamp)
     */
    equal(asset) {
        return asset
            && this.fileChecksum === asset.fileChecksum
            && this.fileType.equal(asset.fileType)
            && this.size === asset.size
            && this.withinRange(this.modified, asset.modified, 1000);
    }
    /**
     * Should only be called on a 'remote' entity. Will apply the local entity's properties to the remote one
     * @param _localEntity - The local entity
     * @returns This object with the applied properties
     */
    apply(_localEntity) {
        return this;
    }
    /**
     *
     * @returns The full asset file path under the provided directory
     */
    getAssetFilePath() {
        return path.format({
            dir: path.join(Resources.manager().dataDir, this.zone === Zones.Primary ? PRIMARY_ASSET_DIR : SHARED_ASSET_DIR),
            name: this.getAssetFilename(),
        });
    }
    /**
     *
     * @returns A filename safe-encoded UUID of this instance with the correct file extension
     */
    getAssetFilename() {
        return path.format({
            name: Buffer.from(this.fileChecksum, `base64`).toString(`base64url`), // Since checksum seems to be base64 encoded
            ext: this.fileType.getExtension(),
        });
    }
    /**
     *
     * @returns The human readable / pretty printed filename of this asset, based on the filename of the original file imported.
     */
    getPrettyFilename() {
        return path.format({
            name: this.origFilename + (this.assetType === AssetType.EDIT ? `-edited` : ``) + (this.assetType === AssetType.LIVE ? `-live` : ``),
            ext: this.fileType.getExtension(),
        });
    }
    /**
     *
     * @returns The UUID of this instance
     */
    getUUID() {
        return this.fileChecksum;
    }
    /**
     * Verifies that the object representation matches the given file
     * @returns True if this file exists and matches this object representation
     * @throws An error, if verification fails
     */
    async verify() {
        let fileStat;
        const filePath = this.getAssetFilePath();
        try {
            fileStat = await fs.stat(filePath);
        }
        catch (err) {
            throw new iCPSError(LIBRARY_ERR.ASSET_NOT_FOUND)
                .addCause(err)
                .addMessage(filePath);
        }
        if (fileStat.size !== this.size) {
            throw new iCPSError(LIBRARY_ERR.ASSET_SIZE)
                .addMessage(`${filePath} size ${fileStat.size}, iCloud ${this.size}`);
        }
        if (!this.withinRange(fileStat.mtimeMs, this.modified, 1000)) {
            throw new iCPSError(LIBRARY_ERR.ASSET_MODIFICATION_TIME)
                .addMessage(`${filePath} modification time ${fileStat.mtimeMs}, iCloud ${this.modified}`)
                .addContext(`out-of-range`, fileStat.mtimeMs - this.modified);
        }
        return true;
    }
    /**
     * Checks if one number is within the range of another number
     * @param x - One number
     * @param y - Other number
     * @param range - Range to check
     * @returns true if within range, false otherwise
     */
    withinRange(x, y, range) {
        return x >= y - range
            && x <= y + range;
    }
    /**
     * Verifies the checksum of this file against the checksum stored in this object
     * This is currently NOT implemented, as the checksum algorithm is unknown.
     * @param file - The read file
     * @returns True if checksum matches
     */
    /*
    private verifyChecksum(file: Buffer): boolean {
        return file !== undefined;
        Const hashes = [
            `BLAKE2b512`,
            `BLAKE2s256`,
            // `MD4`,
            `MD5`,
            `MD6`,
            `MD5-SHA1`,
            //`RIPEMD160`,
            `SHA1`,
            `SHA224`,
            `SHA256`,
            `SHA3-224`,
            `SHA3-256`,
            `SHA3-384`,
            `SHA3-512`,
            `SHA384`,
            `SHA512`,
            `SHA512-224`,
            `SHA512-256`,
            // `SHAKE128`,
            // `SHAKE256`,
            `SM3`,
            // `whirlpool`,
        ];
        const encodings = [
        //    `ascii`,
        //    `utf8`,
        //    `utf-8`,
        //    `utf16le`,
        //    `ucs2`,
        //    `ucs-2`,
            `base64`,
            `base64url`,
            //    `latin1`,
            //    `binary`,
            // `hex`,
        ];
        const key = Buffer.from(this.wrappingKey);// , `base64`);
        hashes.forEach(hash => {
            encodings.forEach(encoding => {
                if (Buffer.isEncoding(encoding)) {
                    try {
                        const hmacChecksum = crypto.createHmac(hash, key)
                            .update(file)
                            .digest()
                            .toString(encoding);

                        const checksum = crypto.createHash(hash)
                            .update(file)
                            .digest()
                            .toString(encoding);

                        if (checksum.includes(this.fileChecksum)) {
                            console.log(`MATCH:     ${checksum} - ${hash}/${encoding}`);
                        } else {
                            console.log(`NO match:  ${checksum} - ${hash}/${encoding}`);
                        }

                        if (hmacChecksum.includes(this.fileChecksum)) {
                            console.log(`MATCH:     ${hmacChecksum} - ${hash}/${encoding}`);
                        } else {
                            console.log(`NO match:  ${hmacChecksum} - ${hash}/${encoding}`);
                        }
                    } catch (error) {
                        console.log(`Problem with ${encoding} + ${hash}: ${error.message}`);
                    }
                }
            });
        });
    } */
    /**
     *
     * @returns A display name for this instance
     */
    getDisplayName() {
        return this.fileChecksum;
    }
}
//# sourceMappingURL=asset.js.map
//# debugId=3c67d18c-90e9-fda4-fb0f-b90fa94f8068