;!function(){try{var k="_btDebugIds",u="undefined",v="08a0deab-c659-d840-c40e-5452e405a0a0",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
import { iCPSError } from "../../../app/error/error.js";
import { LIBRARY_ERR } from "../../../app/error/error-codes.js";
/**
 * Mapping of backend provided filetype description (key) and actual file extension (value)
 * Incomplete list, due to lack of documentation
 */
const EXT = {
    'public.png': `png`,
    'public.mpeg-4': `mp4`,
    'public.jpeg': `jpeg`,
    'com.apple.quicktime-movie': `mov`,
    'public.heic': `heic`,
    'public.image': `HEIC`,
    'com.sony.arw-raw-image': `arw`,
    'org.webmproject.webp': `webp`,
    'com.compuserve.gif': `gif`,
    'com.adobe.raw-image': `dng`,
    'public.tiff': `tiff`,
    'public.jpeg-2000': `jp2`,
    'com.truevision.tga-image': `tga`,
    'com.sgi.sgi-image': `sgi`,
    'com.adobe.photoshop-image': `psd`,
    'public.pbm': `pbm`,
    'public.heif': `heif`,
    'com.microsoft.bmp': `bmp`,
    'public.mpeg': `mpg`,
    'com.apple.m4v-video': `m4v`,
    'public.3gpp': `3gp`,
    'public.mpeg-2-video': `m2v`,
    'com.fuji.raw-image': `raf`,
    'com.canon.cr2-raw-image': `cr2`,
    'com.panasonic.rw2-raw-image': `rw2`,
    'com.nikon.nrw-raw-image': `nrw`,
    'com.pentax.raw-image': `pef`,
    'com.nikon.raw-image': `nef`,
    'com.olympus.raw-image': `orf`,
    'public.avi': `avi`,
    'com.adobe.pdf': `pdf`,
    'public.avchd-mpeg-2-transport-stream': `mts`,
    'com.adobe.illustrator.ai-image': `ai`,
    'com.canon.cr3-raw-image': `cr3`,
    'com.olympus.or-raw-image': `orf`,
    'public.item': `largeThumbnail`,
    'public.mpo-image': `mpo`,
    'com.dji.mimo.pano.jpeg': `jpg`,
    'public.avif': `avif`,
    'com.apple.m4a-audio': `m4a`,
    'com.canon.crw-raw-image': `crw`,
    'public.mp3': `mp3`,
};
/**
 * This class represents the type of file a given asset has
 */
export class FileType {
    /**
     * The descriptor, as provided by the backend
     */
    descriptor;
    /**
     * Constructs a new FileType
     * @param descriptor - The descriptor as provided by the backend
     */
    constructor(descriptor) {
        this.descriptor = descriptor;
    }
    /**
     * Creates the file type from the backend
     * @param descriptor - The descriptor as provided by the backend
     * @param ext - The extension as provided by the encoded filename
     * @returns The newly created FileType
     * @throws An iCPSError, if the provided descriptor is unknown to the script
     */
    static fromAssetType(descriptor, ext) {
        if (!EXT[descriptor]) {
            throw new iCPSError(LIBRARY_ERR.UNKNOWN_FILETYPE_DESCRIPTOR)
                .addContext(`extension`, ext)
                .addContext(`descriptor`, descriptor)
                .addMessage(`${descriptor} (with potential extension ${ext})`);
        }
        return new FileType(descriptor);
    }
    /**
     * Creates a file type from a file extension
     * @param ext - The file extension of the file
     * @returns The newly created FileType
     * @throws An iCPSError, if the provided extension is unknown to the script
     */
    static fromExtension(ext) {
        if (ext.startsWith(`.`)) {
            ext = ext.substring(1);
        }
        const descriptor = Object.keys(EXT).find(key => EXT[key] === ext);
        if (!descriptor) {
            throw new iCPSError(LIBRARY_ERR.UNKNOWN_FILETYPE_EXTENSION)
                .addContext(`extension`, ext)
                .addMessage(ext);
        }
        return new FileType(descriptor);
    }
    /**
     *
     * @returns The FileType as a file extension
     */
    getExtension() {
        return `.` + EXT[this.descriptor];
    }
    /**
     * Compares the given FileType to this instance
     * @param fileType - A FileType to compare this instance to
     * @returns True, if the given FileType is equal to this instance
     */
    equal(fileType) {
        return fileType && this.descriptor === fileType.descriptor;
    }
}
//# sourceMappingURL=file-type.js.map
//# debugId=08a0deab-c659-d840-c40e-5452e405a0a0