;!function(){try{var k="_btDebugIds",u="undefined",v="b8c1bc48-6495-ccf2-f2b5-4c6d19c3e387",a=function(x){try{x[k]=x[k]||{};x[k][n]=v}catch{}},n=(new Error).stack;n&&(u!=typeof window?a(window):u);n&&(u!=typeof global?a(global):u);n&&(u!=typeof self?a(self):u);n&&(u!=typeof globalThis?a(globalThis):u)}catch{}}();
/**
 * This file holds information relevant to networking, as well as type definitions of the expected responses
 */
import { jsonc } from "jsonc";
import { Resources } from "./main.js";
/**
 * Hard coded client id, extracted from previous requests
 */
export const CLIENT_ID = `d39ba9916b7251055b22c7f910e2ea796ee65e98b2ddecea8f5dde8d9d1a815d`;
/**
 * User Agent this CLI is using. Emulating a Firefox Browser
 */
export const USER_AGENT = `Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/130.0.0.0 Safari/537.36`;
/**
 * Client information shared with the iCloud backend based on the user agent
 */
export const CLIENT_INFO = jsonc.stringify({
    U: USER_AGENT,
    L: `en-US`,
    Z: `GMT+01:00`,
    V: `1.1`,
    F: ``,
});
/**
 * Keys for dynamic header values
 */
export const HEADER_KEYS = {
    SCNT: `scnt`,
    SESSION_ID: `X-Apple-ID-Session-Id`,
    COOKIE: `Cookie`,
};
/**
 * Keys for dynamic cookie values
 */
export const COOKIE_KEYS = {
    AASP: `aasp`,
    X_APPLE: `X-APPLE-`,
    PCS_PHOTOS: `X-APPLE-WEBAUTH-PCS-Photos`,
    PCS_SHARING: `X-APPLE-WEBAUTH-PCS-Sharing`,
};
/**
 * List of endpoints used in this application
 */
export const ENDPOINTS = {
    /**
     * Authentication endpoints needed to acquire the session secret and two trust token
     */
    AUTH: {
        BASE: `https://idmsa.apple.com/appleauth/auth`,
        PATH: {
            SIGNIN: {
                LEGACY: `/signin`,
                INIT: `/signin/init`,
                COMPLETE: `/signin/complete`,
            },
            MFA: {
                DEVICE_RESEND: `/verify/trusteddevice`,
                DEVICE_ENTER: `/verify/trusteddevice/securitycode`,
                PHONE_RESEND: `/verify/phone`,
                PHONE_ENTER: `/verify/phone/securitycode`,
                /**
                 * Security key endpoints:
                 * SECURITY_KEY_ENTER: '/verify/security/key'
                 * Payload:
                 * \{
                 *      "challenge":"43-character-challenge",
                 *      "clientData":"[redacted]",
                 *      "signatureData":"[redacted]",
                 *      "authenticatorData":"[redacted]",
                 *      "userHandle":"[redacted]",
                 *      "credentialID":"[redacted]",
                 *      "rpId":"apple.com"
                 * \}
                 */
            },
            TRUST: `/2sv/trust`,
        },
    },
    /**
     * Setup endpoints needed to acquire cookies and Photos URL
     */
    SETUP: {
        BASE() {
            return `https://setup.${Resources.network().iCloudRegionUrl()}`;
        },
        PATH: {
            ACCOUNT_LOGIN: `/setup/ws/1/accountLogin`,
            LOGOUT: `/setup/ws/1/logout`,
            REQUEST_PCS: `/setup/ws/1/requestPCS`,
        },
    },
    /**
     * Photos endpoints needed to access the photos library
     */
    PHOTOS: {
        /**
         * Base URL for photos requests is dynamic - the path is static
         */
        BASE_PATH: `/database/1/com.apple.photos.cloud/production`,
        AREAS: {
            PRIVATE: `/private`,
            SHARED: `/shared`,
        },
        PATH: {
            QUERY: `/records/query`,
            MODIFY: `/records/modify`,
            ZONES: `/changes/database`,
        },
    },
};
//# sourceMappingURL=network-types.js.map
//# debugId=b8c1bc48-6495-ccf2-f2b5-4c6d19c3e387